/***************************************************************
From the book "Motif Programming, The Essentials...and More"
by Marshall Brain.
Published by Digital Press, ISBN 1-55558-089-0.
To order the book, call 1-800-DIGITAL and ask for EY-J816E-DP.

Copyright 1992, by Digital Equipment Corp.

This code demonstrates the creation of a 3 font font list,
and the displaying of an xmstring that uses that font list.
***************************************************************/

/* xmstring.c */

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>

#define SIZE      100

XtAppContext context;

GC gc;
Widget toplevel;
Widget drawing_area;
XmString string;
XmFontList fontlist=NULL;

void setup_gc()
/* set up the graphics context. */
{
    int foreground,background;
    XGCValues vals;
    XFontStruct *font=NULL;
    Arg al[10];
    int ac;

    /* Set the default font in the GC  a neccessary step. */
    font=XLoadQueryFont(XtDisplay(drawing_area),"fixed");

    /* get the current fg and bg colors. */
    ac=0;
    XtSetArg(al[ac],XmNforeground,&foreground); ac++;
    XtSetArg(al[ac],XmNbackground,&background); ac++;
    XtGetValues(drawing_area,al,ac);

    /* create the gc. */
    vals.foreground = foreground;
    vals.background = background;
    vals.font=font->fid; /* The XFontStruct contains a field named
                            "fid" of type Font, as expect by the GC. */
    gc=XtGetGC(drawing_area,GCForeground | GCBackground | GCFont ,&vals);
}

void exposeCB(w,client_data,call_data)
    Widget w;
    XtPointer client_data; 
    XtPointer call_data;
/* called whenever drawing area is exposed. */
{
    XmStringDraw(XtDisplay(drawing_area),XtWindow(drawing_area),
        fontlist,string,gc,10,10,1000,XmALIGNMENT_BEGINNING,
        XmSTRING_DIRECTION_L_TO_R,NULL);
}

void setup_string_and_fontlist()
/* create the XmString and the associated fontlist. */
{
    XFontStruct *font=NULL;
    char *namestring=NULL;
    XmString s1,s2,s3,sep,t1,t2,t3;

    /* Place three different fonts in the fontlist. */
    namestring="*times*24*";        /* you may need to change this */
    font=XLoadQueryFont(XtDisplay(toplevel),namestring);
    fontlist = XmFontListCreate(font,(XmStringCharSet)"charset1");

    namestring="6x10";              /* you may need to change this */
    font=XLoadQueryFont(XtDisplay(toplevel),namestring);
    fontlist = XmFontListAdd(fontlist,font,(XmStringCharSet)"charset2");

    namestring="*helvetica*14*";    /* you may need to change this */
    font=XLoadQueryFont(XtDisplay(toplevel),namestring);
    fontlist = XmFontListAdd(fontlist,font,(XmStringCharSet)"charset3");

    /* create three segments in an XmString and tie them to different
       fonts in the fontlists with corresponding charsets. */
    s1=XmStringCreate("first",(XmStringCharSet)"charset1"); 
    s2=XmStringCreate("second",(XmStringCharSet)"charset2");
    s3=XmStringCreate("third",(XmStringCharSet)"charset3");
    sep=XmStringSeparatorCreate();

    /* build the XmString. */
    t1=XmStringConcat(s1,sep);
    t2=XmStringConcat(t1,s2);
    t3=XmStringConcat(t2,sep);
    string=XmStringConcat(t3,s3);

    /* deallocate memory. */
    XmStringFree(s1);
    XmStringFree(s2);
    XmStringFree(s3);
    XmStringFree(sep);
    XmStringFree(t1);
    XmStringFree(t2);
    XmStringFree(t3);
}

main(argc,argv)
    int argc;
    char *argv[];
{
    Arg al[10];
    int ac;

    /* create the toplevel shell */
    toplevel = XtAppInitialize(&context,"",NULL,0,&argc,argv,
        NULL,NULL,0);

    /* set window size. */
    ac=0;
    XtSetArg(al[ac],XmNheight,SIZE); ac++;
    XtSetArg(al[ac],XmNwidth,SIZE); ac++;
    XtSetValues(toplevel,al,ac);

    /* create a drawing area widget. */
    ac=0;
    drawing_area=XmCreateDrawingArea(toplevel,"drawing_area",al,ac);
    XtManageChild(drawing_area);
    XtAddCallback(drawing_area,XmNexposeCallback,exposeCB,NULL);

    setup_gc();

    setup_string_and_fontlist();

    XtRealizeWidget(toplevel);
    XtAppMainLoop(context);
}
