/***************************************************************
From the book "Motif Programming, The Essentials...and More"
by Marshall Brain.
Published by Digital Press, ISBN 1-55558-089-0.
To order the book, call 1-800-DIGITAL and ask for EY-J816E-DP.

Copyright 1992, by Digital Equipment Corp.

This code demonstrates the use of a selection box dialog.
***************************************************************/

/* selection.c */

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/SelectioB.h>
#include <Xm/List.h>

#define OK        1
#define CANCEL    2

XtAppContext context;
XmStringCharSet char_set=XmSTRING_DEFAULT_CHARSET;

Widget toplevel, button, dialog;
char *animals[]={"cat","dog","cow","goat","horse","mouse","pig",
                 "sheep","rat","donkey","elephant","squirrel"};

void dialogCB(w,client_data,call_data)
    Widget w;
    int client_data;
    XmSelectionBoxCallbackStruct *call_data;
/* callback function for the dialog box. */
{
    char *s;

    switch (client_data)
    {
        case OK:
            /* get the string from the event structure. */
            XmStringGetLtoR(call_data->value,char_set,&s);
            printf("string='%s'\n",s);
            XtFree(s);
            break;
        case CANCEL:
            printf("CANCEL selected\n");
            break;
    }
    XtUnmanageChild(w);
}

void buttonCB(w,client_data,call_data)
    Widget w;
    XtPointer client_data;
    XmPushButtonCallbackStruct *call_data;
/* callback function for the pushbutton */
{
    Arg al[10];
    int ac;
    Widget list;
    int list_cnt;
    XmString s;

    /* Add items to selection box list. */
    list=XmSelectionBoxGetChild(dialog, XmDIALOG_LIST);
    XmListDeleteAllItems(list);
    for (list_cnt=0; list_cnt<XtNumber(animals); list_cnt++)
    {
        s=XmStringCreate(animals[list_cnt],char_set);
        XmListAddItem(list,s,0);
        XmStringFree(s);
    }

    XtManageChild(dialog);
}

void main(argc,argv)
    int argc; 
    char *argv[];
{
    Arg al[10];
    int ac;

    /* create the toplevel shell */
    toplevel = XtAppInitialize(&context,"",NULL,0,&argc,argv,NULL,NULL,0);

    /* Note that the selection box dialog is not created here. 
       See the buttonCB function for an explanation. */

    /* create the pushbutton */
    ac=0;
    XtSetArg(al[ac],XmNlabelString, 
        XmStringCreate("Push Me",char_set)); ac++;
    button=XmCreatePushButton(toplevel,"button",al,ac);
    XtManageChild(button);
    XtAddCallback (button, XmNactivateCallback, buttonCB, NULL);

    /* create the selection box widget */
    ac = 0;
    /* the following line is commented out to make a point. Read          
    more about it in the text description.                       */
    /* XtSetArg(al[ac],XmNautoUnmanage,False); ac++;             */       
    XtSetArg(al[ac],XmNmustMatch,True); ac++;
    XtSetArg(al[ac],XmNselectionLabelString,
        XmStringCreateLtoR("Pick an animal. ",char_set));  ac++;
    dialog=XmCreateSelectionDialog(toplevel,"dialog",al,ac);
    XtAddCallback(dialog,XmNokCallback,dialogCB,OK);
    XtAddCallback(dialog,XmNcancelCallback,dialogCB,CANCEL);
    XtUnmanageChild(XmSelectionBoxGetChild(dialog,
        XmDIALOG_HELP_BUTTON));

    XtRealizeWidget(toplevel);
    XtAppMainLoop(context);
}
